/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Object management
*  SCOPE :              Object management
*  FILE :               obj.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __obj_HXX
#define __obj_HXX

/* IMPORTS *******************************************************************/

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#ifndef __define_H
#include "define.h"
#endif

#ifndef __config_H
#include "config.h"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

/*****************************************************************************/

struct ObjHead {
} ;


// Group attribute for alignement reasons
class Bloc {

  protected :

  short _next ;       // position of the next free bloc in the free list 
  short _classId ;    // class identifier of the object
                      // 0 = free slot
		      // 1 = untyped object
		      // 2... = user defined types
  
  public :

  OID     _oid ;      // store the absolute oid for an object

  Bloc() : _oid(YOO_DummyValue) {} ;  
  int getSlotSize() ;     // return the number of slots occupied by the bloc
  void  setSlotSize(int); // Set the size of the bloc in slot number  
  ObjectX* object() ;     // Return the object corresponding to the bloc  
  int isFree() ;          // Return TRUE if the bloc is free
  void setFree(int ss) ;  // Set the bloc as free and set size in slot number  
  int isTyped() ;         // Return TRUE if the object has a type  
  int getClass() ;        // Return ClassId of the object  
  void setClass(int cl) ; // Set ClassId of the object
} ;


class ObjectX : public Bloc {
    
  public :

  ObjectX() {} ;  
  void* operator new(size_t);      // Standard allocation of an object   
  void* operator new(size_t, Cluster*, int clId, Page* p = NULL) ;  
  void* operator new(size_t, ObjectX* prox,int clId) ;  
  void* operator new(size_t,void*);// tips to reinitialise object at load time
  void operator delete(void* );    // delete an object in DB  
  OID& getOid() ;                  // Return the absolute oid of the object  
  Page* getPage() ;                // Return the page of the object	  
  Cluster* getCluster() ;          // Return cluster of the object  
  ObjectX* changeSize(int ns) ;    // Change object size => Return new address
  int isTemporary() ;              // Return TRUE if the object is temporary  
  int isPersistent() ;             // Return TRUE if the object is persistent  
  void initObj(OID, int sizeSlot); // Initialise a new object
  int objsize() ;                  // Return size of the object
} ;


inline int Bloc::getSlotSize() { 
  return this->_oid.size() ; 
}

inline void  Bloc::setSlotSize(int s) { 
  this->_oid.setsize(s) ; 
} 

inline ObjectX* Bloc::object() { 
  return (ObjectX*)this ; 
}

inline int Bloc::isFree() { 
  return (this->_classId == FREEBLK) ; 
}

inline void Bloc::setFree(int ss) { 
  this->_classId = FREEBLK ; 
  this->setSlotSize(ss) ; 
}

inline int Bloc::isTyped() { 
  return this->_classId >= FIRST_TYPE ; 
}

inline int Bloc::getClass() { 
  return this->_classId ; 
}

inline void Bloc::setClass(int clId) { 
  this->_classId = clId ; 
}

inline OID& ObjectX::getOid() {
  return this->_oid ; 
}
  
inline void* ObjectX::operator new(size_t, void* ptr) {
  return ptr ;
}

#endif
